###################### File mefista.R ################################
# mefista stands for MEasures with FIxed mass' STeepest Ascend/descend
# and contains a code in R realising the steepest descent with
# variable step size for optimization problems in the space of
# positive measures with fixed total mass.
# Theoretical background can be found in I. Molchanov, S. Zuyev
# "Steepest Descent Algorithms in Space of Measures" Statistics
# and Computing,   12, 115-123, 2002
######################################################################
# Version: 2.1
# Date:    27th of November 2011
# Authors: 
#   Ilya Molchanov (ilya.molchanov@stat.unibe.ch)
#   Sergei Zuyev   (sergei@stams.strath.ac.uk)
# Licence: GPL-2
######################################################################
####################### The code begins here #########################


is.optim.mefista<-function(tol,mu,g)
 { # Function checking if g is minimal and constant within error tol
   # on support of mu
   if(length(mu)!=length(g))
     stop("The lists should be of equal length")
   t0<-(tol*sum(mu)/length(mu))
   t.a <- max(g)-min(g)
   t.o <- max(g[mu>t0])-min(g)
   if(t.a==0){t.div <- 0}
   else {t.div <- t.o/t.a}
   return(list(opt=(t.div < tol),var=t.div))
 }

put.mass<-function(m,mu,g,uni=FALSE)
 { # Function putting  mass m to where g is minimal
 if(length(mu)!=length(g)) stop("The lists should be of equal length")
 if(uni)
   {
     l<-length(g[g==min(g)]) # the number of min elements in g
     mu[g==min(g)]<-mu[g==min(g)]+m/l
                             # distributing mass m uniformly over those mu
                             # where g is minimal
   }
 else
   mu[g==min(g)][[1]]<-mu[g==min(g)][[1]]+m
                # putting  mass m in the first found mu where g is minimal
 return(mu)}

take.mass<-function(m,mu,g) 
 { # Function taking mass m from those mu where g is maximal
   len<-length(mu)
   if(len!=length(g))
     stop("The lists should be of equal length")
   r<-order(-g) # indexes of g in decreasing order
   cum<-array(1,len)
   cum[r]<-cumsum(mu[r])
   rest<-m-sum(mu[cum<=m])
   mu[cum<=m]<-0
   ind<-r[cum[r]>m][1] # index of the atom that will not be completely taken
   mu[ind]<-mu[ind]-rest
   return(mu)
 }

go.steep<-function(object, ...)
  {
    if(is.null(class(object))) class(object)<-data.class(object)
    UseMethod("go.steep")
  }

go.steep.mefista<-function(mef,obj.func.name=mef$obj.func$name,
                     grad.func.name=mef$grad.func$name,
                     minim=mef$minim,fval=mef$fval,
                     grad=mef$grad,tol=mef$param$tol,st.size=mef$st.size,
                     algo=mef$param$algo,No.steps=100,space=mef$param$space,
                     details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                     draw.style=4,xgrid=mef$param$xgrid,ygrid=mef$param$ygrid,
                     tmp.file=mef$param$file,alpha=mef$param$alpha,
                     base=mef$param$base,uni=mef$param$uni,
		     multi.steps=mef$multi.steps,
		     reach.only=mef$reach.only,
                     true.m=mef$true.m,nlevels=15,ncolors=0,...)
  {
    ar<-c(list(mu=mef$mes,obj.func=mef$obj.func$code,
               grad.func=mef$grad.func$code,
                     obj.func.name=obj.func.name,grad.func.name=grad.func.name,
                     minim=minim,fval=fval,grad=grad,tol=tol,
               st.size=st.size,algo=algo,No.steps=No.steps,space=space,
               details=details,visualize=visualize,
               smallest.st.size=smallest.st.size,
                     draw.style=draw.style,xgrid=xgrid,ygrid=ygrid,
                     tmp.file=tmp.file,alpha=alpha,base=base,uni=uni,
               dim=mef$param$dim,
                     true.m=true.m),
          nlevels-nlevels,ncolors=ncolors,mef$other,...=...)
    do.call("go.steep.default",ar)
  }

go.steep.default<-function(mu,obj.func,grad.func,
                           obj.func.name=NA,grad.func.name=NA,
                           minim=TRUE,fval=NA,grad=NA,tol=0.01,
                           st.size=0.1*sum(mu),algo=1,No.steps=100,space=NA,
                           details=FALSE,visualize=FALSE,
			   smallest.st.size=0.000001,
                           draw.style=4,xgrid=NA,ygrid=NA,
                           tmp.file=NA,alpha=NA,base=0.5,
		     	   multi.steps=TRUE,reach.only=FALSE,
                           uni=FALSE,dim=NA,true.m=NA,nlevels=15,ncolors=0,...)
{ 
# The main function: minimizes (if minim=TRUE) or maximizes objective
# function with the name obj.func. The gradient function is grad.func,
# fval is the initial value of obj.func if known, true.m is the true
# minimal (maximal) value of obj.func, if known.
# These functions are functions on measures on the state space space,
# space itself
# and possibly some other parameters that should be listed in ... part.
# If space is not supplied then it is taken to be 1:length(mu).
# Algorithm goes at most No.steps steps until the measure mu satisfies
# the necessary optimality condition or reaches tru.m up to error tol.
# Returns object of class mefista.
# A step i consists in changing the current measure mu.cur by a measure eta
# along the gradient grad with the total mass 0 and the total
# variation st.size.cur.
# If the value of objective function in the changed measure do not decrease
# and algo=1 then st.size.cur is changed to base*st.size.cur.
# If algo=2, Armijo method  is used for determination of st.size.cur:
# st.size.cur=base^(k_i)*st.size, where k_i is such that
# (1) obj.func(mu.cur+eta(k,i))-obj.func(mu.cur)<=alpha*int grad d(eta(k,i))
# (2) obj.func(mu.cur+eta(k-1,i))-obj.func(mu.cur)
#                                  > alpha*int grad d(eta(k-1,i))
# Thus alpha should be of order
# (obj.func(mu+eta)-obj.func(mu))/sum(grad*eta), therefore if not given
# it will be so chosen for ||eta||=st.size.
#
# Other parameters:
# tol - the tolerance when deciding if the gradient function is a constant
# on the support of the measure mu. Used in is.optim.mefista() function.
# If details=TRUE, the values of st.size and obj.func(mu) at each step
# are printed.
# If visualize=TRUE the graphs of the measure and the gradient are drawn
# after each step of the algorithm.
# For 2D state space draw.style is 0 for image(), 1 for persp(). If 2, the
# state space will be shown as a vector - useful to see how close to the
# minimum you are.
# In computationally extensive cases it is advisable to set tmp.file to
# a string (name of the file) to store the currenly obtained mu. You then
# will be able to retrieve it with mu<-dget("<name.of.tmp.file>") and
# reuse it if the program interrupts.
# The algorithm adds a positive mass where the gradient function is minimal.
# If there are a few global minima and uni=TRUE then the mass is uniformly
# distributed among them. Otherwize the mass is added in just one point.
# If by some reason you want to check how the classical renormalization
# algorithm as described in Atkinson & Donev "Optimum Experimental Design",
# Oxford 1992 would behave, put algo=0.
#
# First of all, check consisitency of the parameters
if(!is.vector(mu))
  stop("the first parameter should be a vector")
if(!is.function(obj.func))
  stop("'obj.func' should be a function")
if(!is.function(grad.func))
  stop("'grad.func' should be a function")
if(!is.logical(minim))
  stop("'minim' should be logical")
if(!tol>0)
  stop("'tol' should be positive")
if(!st.size>0)
  stop("'st.size' should be positive")
if(st.size>=sum(mu))
  stop("'st.size' should be less than the total mass of the measure")
if(!(algo==0 || algo==1 || algo == 2))
  {
    algo<-1
    warning("'algo' should be 0, 1 or 2: algorithm 1 will be used")
  }
if(!No.steps>0)
  stop("'No.steps' should be positive")
if(!is.logical(details))
  stop("'details' should be logical")
if(!is.logical(visualize))
  stop("'visualize' should be logical")
if(!smallest.st.size>0)
  stop("'smallest.st.size' should be positive")
if(!is.character(tmp.file) && !is.na(tmp.file))
  stop("'tmp.file' should be a character string")
if(!is.na(alpha) && !(alpha>0 && alpha<1))
  stop("'alpha' should be in (0,1)")
if(!(base>0 && base<1))
  stop("'base' should be in (0,1)")



if(is.na(obj.func.name) || !is.character(obj.func.name))
    obj.func.name<-paste("'",deparse(substitute(obj.func)),"'",sep="")
    
if(is.na(grad.func.name) || !is.character(grad.func.name))
  grad.func.name<-paste("'",deparse(substitute(grad.func)),"'",sep="")
if(all(is.na(space)))
  {
    if(all(is.na(xgrid))||all(is.na(ygrid)))
      {
        space<-as.matrix(1:length(mu))
        warning(
            paste("State space is not supplied - considered to be 1:length(",
                  deparse(substitute(mu)),").",sep=""))
      }
    else
      space<-as.matrix(expand.grid(xgrid,ygrid))
  }

if(!is.matrix(space)) space<-as.matrix(space)
if(is.na(dim))  dim<-length(space[1,])
if(length(mu)!=length(space[,1]))
   stop("Lengths of the measure and the state space do not match")
min.ind<-1
if (!minim)
  min.ind<- -1 # indicator of minimization problem
# Getting the value of the goal function
#	
if(is.na(fval)){F0<-min.ind*obj.func(mu,space,...)}
else{F0<-min.ind*fval}
F0.init<-min.ind*F0
###Fval<-c(F0) ### values of obj.func
###Steps<-c(st.size) ### step sizes
# Tolerance of the support of measure
tol.supp<-(tol/length(mu))*sum(mu)
# getting the gradient grad
if(all(is.na(grad)))
  grad<-grad.func(mu,space,...)
grad<-min.ind*grad
# 
true.reach <- FALSE	
if(!is.na(true.m)){true.reach <- (abs(F0-true.m)<tol)}
c.o<-is.optim.mefista(tol,mu,grad)
if(reach.only){is.optim<- true.reach}
else{is.optim<- c.o$opt | true.reach}


if (is.optim)
  { # Collect some info at least
    v.n <- c.o$var
    supp<-matrix(space[mu>tol.supp,],ncol=dim)
    mu.supp<-mu[mu>tol.supp]
    No.atoms<-length(mu.supp)
    if(No.atoms==1)
      supp.str<-paste("\nThe obtained measure contains 1 atom:","\n\n",sep="")
    else
      supp.str<-paste("\nThe obtained measure contains ",
                      No.atoms," atoms",
                      "\n(you might consider collapsing close ones):",
                      "\n\n",sep="")
    for (j in 1:No.atoms)
      { 
        if(dim==1)
          coor<-paste("(",as.character(supp[j,1]),")",sep="")
        else
          {
            coor<-"("
            for (k in 1:(dim-1))
              { 
                coor<-paste(coor,as.character(supp[j,k]),",",sep="")
              }
            coor<-paste(coor,as.character(supp[j,dim]),")",sep="")
          }
        supp.str<-paste(supp.str,"mu",coor,"=",format(mu.supp[j],digits=6),
                        "\n",sep="")
      }
    if(true.reach)
      cat("The true extremal value",true.m,"is attained within error tol=",tol)
    else
      cat("The measure already satisfies the optimality condition\n")
    invisible(return(structure(list(mes=mu,
                                obj.func
                                    =list(name=obj.func.name,code=obj.func),
                                grad.func
                                    =list(name=grad.func.name,code=grad.func),
                                    minim=minim,fval=min.ind*F0,
                    grad=min.ind*grad,var.grad=v.n,optim=TRUE,st.size=st.size,
                    atoms=supp.str,No.atoms=No.atoms,tot.steps=0,
		    reach.only=reach.only,multi.steps=multi.steps,
                    true.m=true.m,#call=call.str,
                                   ### Fval=Fval, Steps=Steps,###
                    param=list(tol=tol,algo=algo,init.st.size=st.size,
                    smallest.st.size=smallest.st.size,uni=uni,space=space,
                    xgrid=xgrid,ygrid=ygrid,dim=dim,
                    alpha=alpha,base=base,file=tmp.file),other=list(...)),
                                      class="mefista")))
                                     
  }
i<-1  # number of steps along the gradient
st.size.cur<-st.size # the current step size
too.small.step.size<-FALSE
if (!details) cat("\nSteps done:   ")
if(algo==2) 
  { # Armijo algorithm for step size
    if(is.na(alpha))
      {
        mu2<-put.mass(st.size,mu,grad,uni)
        mu2<-take.mass(st.size,mu2,grad)
        F2<-min.ind*obj.func(mu2,space,...)
        alpha<- min(0.1,abs((F2-F0)/sum(grad*(mu2-mu))))  ### ;-)
      }

    while (i<=No.steps & !is.optim & !too.small.step.size)
      { # while i<=No.steps loop
        mu2<-put.mass(st.size.cur,mu,grad,uni)
        mu2<-take.mass(st.size.cur,mu2,grad)
        F2<-min.ind*obj.func(mu2,space,...)
        F1<-F2
        mu1<-mu2
        anm<-alpha*sum(grad*mu)
        check.arm<- (F2-F0 <= alpha*sum(grad*mu2)-anm)
                                        # Armijo condition (1) for new mu2
        if(!check.arm)
          { 
            while(!check.arm)
              {
                st.size.cur<-st.size.cur*base # decrease the st.size
                if(!details)
                  cat("-") ###
                if (st.size.cur<smallest.st.size)
                  {
                    too.small.step.size<-TRUE
                    break
                  }
                mu2<-put.mass(st.size.cur,mu,grad,uni)
                mu2<-take.mass(st.size.cur,mu2,grad)
                F2<-min.ind*obj.func(mu2,space,...)
                check.arm<- (F2-F0 <= alpha*sum(grad*mu2)-anm)
              }
            F0<-F2
            mu<-mu2
            ###cat("\n")
          }
        else
          {
            bm<-base*sum(mu)
            while(check.arm && st.size.cur <= bm) # the step should not exceed
              # the total mass, however
              {
                F1<-F2
                mu1<-mu2 # remember values for which (1) is true
                st.size.cur<-st.size.cur/base # increase the st.size
                mu2<-put.mass(st.size.cur,mu,grad,uni)
                mu2<-take.mass(st.size.cur,mu2,grad)
                F2<-min.ind*obj.func(mu2,space,...)
                check.arm<- (F2-F0 <= alpha*sum(grad*mu2)-anm)
                if(check.arm && !details)
                  cat("+") ###
              }
            F0<-F1
            mu<-mu1
            ###cat("\n")
          }
        ###Fval<-c(Fval,F0) ### add a new value
        ###Steps<-c(Steps,st.size.cur) ###
        if(!is.na(tmp.file))  # tmp.file is set - write the current measure
          dput(mu,tmp.file)
        grad<-min.ind*grad.func(mu,space,...) # Take a new gradient
        if(!is.na(true.m)){true.reach <- (abs(F0-true.m)<tol)}
        c.o<-is.optim.mefista(tol,mu,grad)
        if(reach.only){is.optim<- true.reach}
	else{is.optim<- c.o$opt | true.reach}
                                        #if TRUE the loop finishes
        if (details)
          {
            v.n <- c.o$var
            cat("Step:",i,"  ","St.size:",st.size.cur,"\t")
            cat("obj.func=",min.ind*F0,"  ","var.grad=",v.n,"\n")
          }
        else
          {
            cat(".")
          }
        if(visualize) 
          {
            draw.mef(mu,min.ind*grad,minim,draw.style=draw.style,
                 space,xgrid,ygrid,nlevels,ncolors)
          }
        i<-i+1
      } # end of while i<=No.steps loop
    i0<-i-1 # For compatibility of output
  }
else
  { # Not Armijo
    i0<-0 # the total number of incremental steps
    while (i<=No.steps & !is.optim & !too.small.step.size)
      { # while i<=No.steps loop
        i0.change<-i0   #remember the step at which the direction changes
        is.smaller<-TRUE   #initialisation of the while-loop below:
        while (is.smaller)
          { # go in the direction grad while obj.func(mu) diminishes
                                        # Now try an incremental step:
            if (algo==0) # renormalization algorithm
              {
                mu.new<-
                  put.mass(st.size.cur/(sum(mu)-st.size.cur),mu,grad,uni)
               # term 1/(1-st.size.cur) makes the total variation
               # of the difference measure equal approximately to 2*st.size
               # for the sake of comparability with algorithm 1 below.
                mu.new<-mu.new/sum(mu.new)
              }
            else    # algorithm 1
              {
                mu.new<-put.mass(st.size.cur,mu,grad,uni)
                mu.new<-take.mass(st.size.cur,mu.new,grad)
              }
            F1<-min.ind*obj.func(mu.new,space,...)
            if (F1<F0)   # obj.func has decreased, update mu
              { 
                mu<-mu.new
                i0<-i0+1 # incremental step was done
                F0<-F1
                if(!details)
                  cat(".")
		is.smaller <- is.smaller & multi.steps
              }
            else  # obj.func has not decreased - end the while loop
              {
                is.smaller<-FALSE                
                if (i0.change==i0) # not only we can not make a step
                  {               # but we have not done any step at all: 
                    st.size.cur<-st.size.cur*base # decrease the st.size 
                    i<-(i-1)		    # do not count this step
                    if(!details)
                      cat("-")
                    if (st.size.cur<smallest.st.size)
                      too.small.step.size<-TRUE
                  }
                else # increase st.size if too many incremental steps
                  # was made with the current st.size
                  {
                    st.size.cur<-st.size.cur*(i0-i0.change)
                    if(!details)
                      cat(paste(rep("+",i0-i0.change-1),sep="",collapse=""))
                  }
              }
          } # end of while(is.smaller) cycle
        ###Fval<-c(Fval,F0) ### add new value
        ###Steps<-c(Steps,st.size.cur) ###
        if(!is.na(tmp.file))  # tmp.file is set - write the current measure
          dput(mu,tmp.file)
        grad<-min.ind*grad.func(mu,space,...) # Take a new gradient
        if(!is.na(true.m))
          true.reach <- (abs(F0-true.m)<tol)
        c.o<-is.optim.mefista(tol,mu,grad)
        if(reach.only){is.optim<-true.reach}
	else{is.optim<-c.o$opt | true.reach}       
                                        #if TRUE the loop finishes
	if (details)
          {
            v.n <- c.o$var
            cat("Step:",i,"  ","St.size:",st.size.cur,"\t")
            cat("obj.func=",min.ind*F0,"  ","var.grad=",v.n,"\n")
          }
        if(visualize) 
          {
            draw.mef(mu,min.ind*grad,minim,draw.style=draw.style,
                 space,xgrid,ygrid,nlevels,ncolors)
          }
        i<-i+1
      } # end of while(i<=No.steps...) loop
  }  # end of Not Armijo part

# The descent is over now. Print the current results
if (!details)
  cat("\n")
if (is.optim) 
  { 
    if(true.reach)
      cat("THE TRUE EXTREMAL VALUE IS REACHED.")
    else
      cat("\n","OPTIMUM IS ATTAINED.","\n")
  }
else
  { 
    if (too.small.step.size)
      {
        cat("\nCOMPUTATION STOPPED - TOO SMALL STEP SIZE:",st.size.cur,"\n")
      }
    cat("\n OPTIMUM IS NOT YET ATTAINED.","\n\n")
    cat("Consider rerunning go.steep() and possibly change the parameters.\n")
  }
# Now generate string to print mu in its support points only
supp<-matrix(space[mu>tol.supp,],ncol=dim)
mu.supp<-mu[mu>tol.supp]
No.atoms<-length(mu.supp)
if(No.atoms==1)
  supp.str<-paste("\nThe obtained measure contains 1 atom:","\n\n",sep="")
else
  supp.str<-paste("\nThe obtained measure contains ",
                No.atoms," atoms",
                "\n(you might consider collapsing close ones):",
                "\n\n",sep="")
for (j in 1:No.atoms)
  { 
    if(dim==1)
      coor<-paste("(",as.character(supp[j,1]),")",sep="")
    else
      {
        coor<-"("
        for (k in 1:(dim-1))
          { 
            coor<-paste(coor,as.character(supp[j,k]),",",sep="")
          }
        coor<-paste(coor,as.character(supp[j,dim]),")",sep="")
      }
    supp.str<-paste(supp.str,"mu",coor,"=",format(mu.supp[j],digits=6),
                    "\n",sep="")
  }
cat("\nValue of",obj.func.name,"was:",F0.init,
    "  now:",min.ind*F0,"\n")
v.n <- c.o$var
if (details)
  {
    if (No.atoms<31 || is.optim)
      {
        cat("\n","Current measure:","\n")
        cat(supp.str)
      }
    else
      {
        cat("\nCurrent measure contains",No.atoms,"atoms: ")
        cat("the output is suppressed." )
        cat("\nTo see the support points use \n\tcat(<mefista_obj>$atoms)\n")
      }
    cat("\nVariation of the gradient on the measure's support =",v.n,"\n")     

  # This may serve as an indication of how close we are to the optimum
#    cat("Total No. of incremental steps made:",i0,"\n")
  }
invisible(return(structure(list(mes=mu,
                                obj.func
                                =list(name=obj.func.name,code=obj.func),
                                grad.func
                                =list(name=grad.func.name,code=grad.func),
                                minim=minim,fval=min.ind*F0,
                                grad=min.ind*grad,var.grad=v.n,
                                optim=is.optim,st.size=st.size.cur,
                                atoms=supp.str,No.atoms=No.atoms,
                                tot.steps=i-1,true.m=true.m,#call=call.str,
				reach.only=reach.only,
				multi.steps=multi.steps,
                                ###Fval=Fval,Steps=Steps,
                                too.small.step.size=too.small.step.size,
                                param=list(tol=tol,algo=algo,
                                init.st.size=st.size,
                                smallest.st.size=smallest.st.size,
                                uni=uni,space=space,
                                xgrid=xgrid,ygrid=ygrid,dim=dim,
                                alpha=alpha,base=base,file=tmp.file),
                                other=list(...)),
                           class="mefista")))
}


print.mefista<-function(mef,...)
  {
    mf<-mef
    if (mf$minim)
      cat("\n   Minimization of function",mf$obj.func$name,"\n")
    else
      cat("\n   Maximization of function",mf$obj.func$name,"\n")
    mef<-unlist(c(format(mef[5],digits=8),mef[8],mef[[17]][1],
                  format(mef[7],digits=5),mef[[17]][2]))
    NextMethod("print")
    invisible(mf)
  }

summary.mefista<-function(mef)
  structure(mef, class=c("sum.mefista",class(mef)))

print.sum.mefista<-function(mef,...)
  {
    if (mef$minim)
      cat("\n    Minimization of function",mef$obj.func$name,
          "in dimension",mef$param$dim,"\n")
    else
      cat("\n    Maximization of function",mef$obj.func$name,
          "in dimension",mef$param$dim,"\n")
    cat("with gradient function",mef$grad.func$name,".\n")
#    cat("\nFunction called:\n")
#    print(mef$call)
    cat("\nAlgorithm",mef$param$algo,"was called with tolerance",mef$param$tol)
    cat("\n\tand initial step size",mef$param$init.st.size)
    cat("\nFinal step size:",mef$st.size,"\n")
    if (mef$optim) 
      {
        if(!is.na(mef$true.m) && abs(mef$fval-mef$true.m)<mef$param$tol)
          cat("TRUE EXTREMUM",mef$true.m,"IS ATTAINED WITHIN ERROR",
              mef$param$tol)
        else
          cat("\n","OPTIMUM IS ATTAINED.","\n")
         if (mef$No.atoms<31)
          {
            cat("\n","OPTIMAL MEASURE:","\n")
            cat(mef$atoms)
          }
        else
          {
            cat("\nOptimal measure contains",mef$No.atoms,"atoms: ")
            cat("the output is suppressed." )
            cat("\nTo see the support points use")
            cat("\n\tcat(<mefista_obj>$atoms)\n")
          }
      }
    else
      { 
        cat("\n","OPTIMUM IS NOT YET ATTAINED.","\n\n")
        cat("Consider rerunning go.steep()")
        cat("\nand possibly change the parameters.","\n")
        if (mef$No.atoms<31)
          {
            cat("\n","Current measure:","\n")
            cat(mef$atoms)
          }
        else
          {
            cat("\nCurrent measure contains",mef$No.atoms,"atoms:")
            cat("\nOutput is suppressed." )
            cat("\nTo see the support points use cat(<mefista_obj>$atoms)")
            #cat("\nTo see the support points use cat(",list(mef),"$atoms)") 
          }
      }
    # Now print some statistics
    cat("\n\nValue of the objective function",mef$obj.func$name,
        ":",mef$fval,"\n")
    cat("\nVariation of the gradient on the measure's support:",mef$var.grad)
    cat("\nTotal No. of steps made during the last call:",mef$tot.steps,"\n")
    invisible(mef)
  }    

plot.mefista<-function(mef,draw.style=NA,space=mef$param$space,xgrid=mef$param$
xgrid,ygrid=mef$param$ygrid,nlevels=15,ncolors=0,...)
  {
    if(mef$param$dim==1 && all(is.na(space)))
      warning("'space' is not supplied - considered to be 1:length(measure)")
    if(mef$param$dim>2)
      warning("space considered to be 1:length(measure)")
    if(mef$param$dim==2 && (all(is.na(xgrid)) || all(is.na(ygrid))))
      {
	if(!all(is.na(space)))
          {
            xgrid <- as.numeric(names(table(space[,1])))
            ygrid <- as.numeric(names(table(space[,2])))
          }
      }
    if(is.na(draw.style))
      {
        draw.style<-0
        if(mef$param$dim==2) draw.style<-4
      }
    draw.mef(mef$mes,mef$grad,mef$minim,draw.style,space,xgrid,ygrid,
         nlevels=nlevels,ncolors=ncolors,...)
    invisible(mef)
  }

draw.mef<-function(mu,g,minim=TRUE,draw.style=0,space=NA,xgrid=NA,ygrid=NA,
                   nlevels=15,ncolors=0,...)
  { # Function plotting mu, g and the minimum of g. For 2D plots xgrid is the
    # sequence of grid points along x-axis, ygrid - along y-axis.
    # draw.style is either
    # 0 for linear plot as 1D sequence
    # 1 for image() with no. of levels equal nlevels
    # 2 for contour() with no. of levels equal nlevels
    # 3 for image() and contour superposed
    # 4 for persp()
    # ncolors - if 0 the default colors are used. 

    par(mfrow=c(2,1))
   
    if (all(is.na(space)))
      {
        space<-1:length(mu)
        draw.style<-0
        dim<-1
        #warning(paste("State space is considered to be 1:lenght(",
        #       deparse(substitute(mu)),").",sep=""))
       }
    else
      {
        dim<-length(space[1,])
        if (dim==1)
          {
            draw.style<-0
          # warning("The only available option for 1D plot is draw.style=0")
          }
        if(dim==2 && draw.style==0)
          space<-1:length(mu)
        if(dim==2 && draw.style!=0)
          {
            xgrid <- as.numeric(names(table(space[,1])))
	    ygrid <- as.numeric(names(table(space[,2])))
          }
        if(dim>2)
          {
            draw.style<-0
            space<-1:length(mu)
      #warning("The only available option for dimensions > 2 is draw.style=0")
          }
      }

    if(draw.style==0)
      {
        plot(space,g,type="l",main="Gradient function",lwd=2,axes=TRUE,
             col="darkred",xlab="",ylab="",...)
        if(minim)
          abline(h=min(g),lwd=2,col="green")
        else
          abline(h=max(g),lwd=2,col="green")
        plot(space,mu,type="h",lwd=2,main="Measure",axes=TRUE,col="red",
             xlab="",ylab="",...)
        abline(h=0)
      }
    
    if (dim==2)
      {
        G<-matrix(g,ncol=length(ygrid))
        MU<-matrix(mu,ncol=length(ygrid))
        if (draw.style==1)
          {
            if(ncolors==0)
              image(xgrid,ygrid,G,...)
            else
              image(xgrid,ygrid,G,col=rainbow(ncolors,start=3/6,end=5/6),...)
            title(main="Gradient function")
            if(ncolors==0)
              image(xgrid,ygrid,MU,...)
            else
              image(xgrid,ygrid,MU,col=rainbow(ncolors,start=3/6,end=5/6),...)
            title(main="Measure")
          }
        if (draw.style==2)
          {
            contour(xgrid,ygrid,G,nlevels=nlevels,...)
            title(main="Gradient function")
            contour(xgrid,ygrid,MU,nlevels=nlevels,...)
            title(main="Measure")
          }
        if (draw.style==3)
          {
            if(ncolors==0)
              image(xgrid,ygrid,G,...)
            else
              image(xgrid,ygrid,G,col=rainbow(ncolors,start=3/6,end=5/6),...)
            contour(xgrid,ygrid,G,nlevels=nlevels,add=TRUE,...)
            title(main="Gradient function")
            if(ncolors==0)
              image(xgrid,ygrid,MU,...)
            else
              image(xgrid,ygrid,MU,col=rainbow(ncolors,start=3/6,end=5/6),...)
            contour(xgrid,ygrid,MU,nlevels=nlevels,add=TRUE,...)
            title(main="Measure")
          }
        if (draw.style==4 & ncolors>0)
          {
            persp(xgrid,ygrid,G,theta=30, phi=30,d=3,
                  col="grey",shade=0.3,...)
            title(main="Gradient function")
            persp(xgrid,ygrid,MU,theta=30, phi=30,d=3,
                  col="cyan",shade=0.3,...)
            title(main="Measure")
          }
        if (draw.style==4 & ncolors==0)
          {
            persp(xgrid,ygrid,G,theta=30, phi=30,d=3,...)
            title(main="Gradient function")
            persp(xgrid,ygrid,MU,theta=30, phi=30,d=3,...)
            title(main="Measure")
          }
      } 
  }


######################## End of mefista.R ################################


